//+------------------------------------------------------------------+
//|                                                wz_30_feature.mq5 |
//|                                  Copyright 2024, MetaQuotes Ltd. |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2024, MetaQuotes Ltd."
#property link      "https://www.mql5.com"
#property version   "1.00"
//+------------------------------------------------------------------+
//| Include                                                          |
//+------------------------------------------------------------------+
#include <Expert\Expert.mqh>
//--- available signals
#include <Expert\Signal\My\SignalWZ_30_A.mqh>
//--- available trailing
#include <Expert\Trailing\TrailingNone.mqh>
//--- available money management
#include <Expert\Money\MoneyFixedMargin.mqh>
//+------------------------------------------------------------------+
//| Inputs                                                           |
//+------------------------------------------------------------------+
//--- inputs for expert
input string Expert_Title               = "wz_30_feature"; // Document name
ulong        Expert_MagicNumber         = 15421;          //
bool         Expert_EveryTick           = false;          //
//--- inputs for main signal
input int    Signal_ThresholdOpen       = 10;             // Signal threshold value to open [0...100]
input int    Signal_ThresholdClose      = 10;             // Signal threshold value to close [0...100]
input double Signal_PriceLevel          = 0.0;            // Price level to execute a deal
input double Signal_StopLevel           = 50.0;           // Stop Loss level (in points)
input double Signal_TakeLevel           = 50.0;           // Take Profit level (in points)
input int    Signal_Expiration          = 4;              // Expiration of pending orders (in bars)
input int    Signal_CMLP_TrainSet       = 260;            // CMLP(260,65,0.0005,5,0.5,...) Training Set Size
input int    Signal_CMLP_Epochs         = 65;             // CMLP(260,65,0.0005,5,0.5,...) Epochs
input double Signal_CMLP_LearningRate   = 0.0005;         // CMLP(260,65,0.0005,5,0.5,...) Learning Rate
input int    Signal_CMLP_DecayEpochSteps = 5;             // CMLP(260,65,0.0005,5,0.5,...) Decay Epoch Steps
input double Signal_CMLP_DecayRateA     = 0.5;            // CMLP(260,65,0.0005,5,0.5,...) Decay Rate A
input double Signal_CMLP_DecayRateB     = 0.5;            // CMLP(260,65,0.0005,5,0.5,...) Decay Rate B
input int    Signal_CMLP_PolynomialPower = 2;             // CMLP(260,65,0.0005,5,0.5,...) Polynomial Power
input double Signal_CMLP_MinLearningRate = 0.0001;        // CMLP(260,65,0.0005,5,0.5,...) Minimum Learning Rate
input double Signal_CMLP_DefaultWeight  = 0.005;          // CMLP(260,65,0.0005,5,0.5,...) Default Weight
input double Signal_CMLP_DefaultBias    = 0.0;            // CMLP(260,65,0.0005,5,0.5,...) Default Bias
input ENUM_ACTIVATION_FUNCTION Signal_CMLP_ActivationType = AF_SIGMOID;  // CMLP(30,6,0.05,10,0.1,2,...) Activation Type
input Enormalize Signal_CMLP_NormalizeType   = NORMALIZE_FEATURE; // CMLP(30,6,0.05,10,0.1,2,...) Batch Normalisation Type
input Elearning Signal_CMLP_LearningType   = LEARNING_ADAPTIVE; // CMLP(30,6,0.05,10,0.1,2,...) Learning Type
input Eadaptive Signal_CMLP_AdaptiveType   = ADAPTIVE_GRADIENT; // CMLP(30,6,0.05,10,0.1,2,...) Adaptive Type
input double Signal_CMLP_NormBeta_H_1   = 0.0;            // CMLP(260,65,0.0005,5,0.5,...) Normalization Beta Hidden 1
input double Signal_CMLP_NormBeta_H_2   = 0.0;            // CMLP(260,65,0.0005,5,0.5,...) Normalization Beta Hidden 2
input double Signal_CMLP_NormBeta_H_3   = 0.0;            // CMLP(260,65,0.0005,5,0.5,...) Normalization Beta Hidden 3
input double Signal_CMLP_NormBeta_H_4   = 0.0;            // CMLP(260,65,0.0005,5,0.5,...) Normalization Beta Hidden 4
input double Signal_CMLP_NormBeta_H_5   = 0.0;            // CMLP(260,65,0.0005,5,0.5,...) Normalization Beta Hidden 5
input double Signal_CMLP_NormBeta_H_6   = 0.0;            // CMLP(260,65,0.0005,5,0.5,...) Normalization Beta Hidden 6
input double Signal_CMLP_NormBeta_H_7   = 0.0;            // CMLP(260,65,0.0005,5,0.5,...) Normalization Beta Hidden 7
input double Signal_CMLP_NormBeta_H_8   = 0.0;            // CMLP(260,65,0.0005,5,0.5,...) Normalization Beta Hidden 8
input double Signal_CMLP_NormGamma_H_1  = 0.0;            // CMLP(260,65,0.0005,5,0.5,...) Normalization Gamma Hidden 1
input double Signal_CMLP_NormGamma_H_2  = 0.0;            // CMLP(260,65,0.0005,5,0.5,...) Normalization Gamma Hidden 2
input double Signal_CMLP_NormGamma_H_3  = 0.0;            // CMLP(260,65,0.0005,5,0.5,...) Normalization Gamma Hidden 3
input double Signal_CMLP_NormGamma_H_4  = 0.0;            // CMLP(260,65,0.0005,5,0.5,...) Normalization Gamma Hidden 4
input double Signal_CMLP_NormGamma_H_5  = 0.0;            // CMLP(260,65,0.0005,5,0.5,...) Normalization Gamma Hidden 5
input double Signal_CMLP_NormGamma_H_6  = 0.0;            // CMLP(260,65,0.0005,5,0.5,...) Normalization Gamma Hidden 6
input double Signal_CMLP_NormGamma_H_7  = 0.0;            // CMLP(260,65,0.0005,5,0.5,...) Normalization Gamma Hidden 7
input double Signal_CMLP_NormGamma_H_8  = 0.0;            // CMLP(260,65,0.0005,5,0.5,...) Normalization Gamma Hidden 8
input double Signal_CMLP_NormBeta_O_1   = 0.0;            // CMLP(260,65,0.0005,5,0.5,...) Normalization Beta Output 1
input double Signal_CMLP_NormGamma_O_1  = 0.0;            // CMLP(260,65,0.0005,5,0.5,...) Normalization Gamma Output 1
input double Signal_CMLP_Weight         = 1.0;            // CMLP(260,65,0.0005,5,0.5,...) Weight [0...1.0]
//--- inputs for money
input double Money_FixMargin_Percent    = 10.0;           // Percentage of margin
//+------------------------------------------------------------------+
//| Global expert object                                             |
//+------------------------------------------------------------------+
CExpert ExtExpert;
//+------------------------------------------------------------------+
//| Initialization function of the expert                            |
//+------------------------------------------------------------------+
int OnInit()
{
//--- Initializing expert
   if(!ExtExpert.Init(Symbol(), Period(), Expert_EveryTick, Expert_MagicNumber))
   {  //--- failed
      printf(__FUNCTION__ + ": error initializing expert");
      ExtExpert.Deinit();
      return(INIT_FAILED);
   }
//--- Creating signal
   CExpertSignal *signal = new CExpertSignal;
   if(signal == NULL)
   {  //--- failed
      printf(__FUNCTION__ + ": error creating signal");
      ExtExpert.Deinit();
      return(INIT_FAILED);
   }
//---
   ExtExpert.InitSignal(signal);
   signal.ThresholdOpen(Signal_ThresholdOpen);
   signal.ThresholdClose(Signal_ThresholdClose);
   signal.PriceLevel(Signal_PriceLevel);
   signal.StopLevel(Signal_StopLevel);
   signal.TakeLevel(Signal_TakeLevel);
   signal.Expiration(Signal_Expiration);
//--- Creating filter CSignalCMLP
   CSignalCMLP *filter0 = new CSignalCMLP;
   if(filter0 == NULL)
   {  //--- failed
      printf(__FUNCTION__ + ": error creating filter0");
      ExtExpert.Deinit();
      return(INIT_FAILED);
   }
   signal.AddFilter(filter0);
//--- Set filter parameters
   filter0.TrainSet(Signal_CMLP_TrainSet);
   filter0.Epochs(Signal_CMLP_Epochs);
   filter0.LearningRate(Signal_CMLP_LearningRate);
   filter0.ActivationType(Signal_CMLP_ActivationType);
   filter0.NormalizeType(Signal_CMLP_NormalizeType);
   filter0.LearningType(Signal_CMLP_LearningType);
   filter0.AdaptiveType(Signal_CMLP_AdaptiveType);
   filter0.DecayEpochSteps(Signal_CMLP_DecayEpochSteps);
   filter0.DecayRateA(Signal_CMLP_DecayRateA);
   filter0.DecayRateB(Signal_CMLP_DecayRateB);
   filter0.PolynomialPower(Signal_CMLP_PolynomialPower);
   filter0.MinLearningRate(Signal_CMLP_MinLearningRate);
   filter0.DefaultWeight(Signal_CMLP_DefaultWeight);
   filter0.DefaultBias(Signal_CMLP_DefaultBias);
   filter0.NormBeta_H_1(Signal_CMLP_NormBeta_H_1);
   filter0.NormBeta_H_2(Signal_CMLP_NormBeta_H_2);
   filter0.NormBeta_H_3(Signal_CMLP_NormBeta_H_3);
   filter0.NormBeta_H_4(Signal_CMLP_NormBeta_H_4);
   filter0.NormBeta_H_5(Signal_CMLP_NormBeta_H_5);
   filter0.NormBeta_H_6(Signal_CMLP_NormBeta_H_6);
   filter0.NormBeta_H_7(Signal_CMLP_NormBeta_H_7);
   filter0.NormBeta_H_8(Signal_CMLP_NormBeta_H_8);
   filter0.NormGamma_H_1(Signal_CMLP_NormGamma_H_1);
   filter0.NormGamma_H_2(Signal_CMLP_NormGamma_H_2);
   filter0.NormGamma_H_3(Signal_CMLP_NormGamma_H_3);
   filter0.NormGamma_H_4(Signal_CMLP_NormGamma_H_4);
   filter0.NormGamma_H_5(Signal_CMLP_NormGamma_H_5);
   filter0.NormGamma_H_6(Signal_CMLP_NormGamma_H_6);
   filter0.NormGamma_H_7(Signal_CMLP_NormGamma_H_7);
   filter0.NormGamma_H_8(Signal_CMLP_NormGamma_H_8);
   filter0.NormBeta_O_1(Signal_CMLP_NormBeta_O_1);
   filter0.NormGamma_O_1(Signal_CMLP_NormGamma_O_1);
   filter0.Weight(Signal_CMLP_Weight);
//--- Creation of trailing object
   CTrailingNone *trailing = new CTrailingNone;
   if(trailing == NULL)
   {  //--- failed
      printf(__FUNCTION__ + ": error creating trailing");
      ExtExpert.Deinit();
      return(INIT_FAILED);
   }
//--- Add trailing to expert (will be deleted automatically))
   if(!ExtExpert.InitTrailing(trailing))
   {  //--- failed
      printf(__FUNCTION__ + ": error initializing trailing");
      ExtExpert.Deinit();
      return(INIT_FAILED);
   }
//--- Set trailing parameters
//--- Creation of money object
   CMoneyFixedMargin *money = new CMoneyFixedMargin;
   if(money == NULL)
   {  //--- failed
      printf(__FUNCTION__ + ": error creating money");
      ExtExpert.Deinit();
      return(INIT_FAILED);
   }
//--- Add money to expert (will be deleted automatically))
   if(!ExtExpert.InitMoney(money))
   {  //--- failed
      printf(__FUNCTION__ + ": error initializing money");
      ExtExpert.Deinit();
      return(INIT_FAILED);
   }
//--- Set money parameters
   money.Percent(Money_FixMargin_Percent);
//--- Check all trading objects parameters
   if(!ExtExpert.ValidationSettings())
   {  //--- failed
      ExtExpert.Deinit();
      return(INIT_FAILED);
   }
//--- Tuning of all necessary indicators
   if(!ExtExpert.InitIndicators())
   {  //--- failed
      printf(__FUNCTION__ + ": error initializing indicators");
      ExtExpert.Deinit();
      return(INIT_FAILED);
   }
//--- ok
   return(INIT_SUCCEEDED);
}
//+------------------------------------------------------------------+
//| Deinitialization function of the expert                          |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
{  ExtExpert.Deinit();
}
//+------------------------------------------------------------------+
//| "Tick" event handler function                                    |
//+------------------------------------------------------------------+
void OnTick()
{  ExtExpert.OnTick();
}
//+------------------------------------------------------------------+
//| "Trade" event handler function                                   |
//+------------------------------------------------------------------+
void OnTrade()
{  ExtExpert.OnTrade();
}
//+------------------------------------------------------------------+
//| "Timer" event handler function                                   |
//+------------------------------------------------------------------+
void OnTimer()
{  ExtExpert.OnTimer();
}
//+------------------------------------------------------------------+
